package com.patzn.lims.equip.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.patzn.lims.core.PtConstants;
import com.patzn.lims.core.api.PtResult;
import com.patzn.lims.core.web.ServiceController;
import com.patzn.lims.equip.entity.LmsEquipCheckDetail;
import com.patzn.lims.equip.entity.LmsEquipFile;
import com.patzn.lims.equip.service.ILmsEquipCheckDetailService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.List;

/**
 * <p>
 * 期间核查详情信息表 前端控制器
 * </p>
 *
 * @author SunYunGang
 * @since 2019-01-05
 */
@Api(description = "期间核查详情信息表")
@RestController
@RequestMapping("/v1/equip_check_detail")
public class LmsEquipCheckDetailController extends ServiceController<ILmsEquipCheckDetailService, LmsEquipCheckDetail>  {

    @Autowired
    private ILmsEquipCheckDetailService lmsEquipCheckDetailService;


    @ApiOperation(value = "分页列表", notes = "分页查询数据列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = PtConstants.PAGE_PAGE, value = "请求数据的页码", required = true, paramType = "query", dataType = "int"),
            @ApiImplicitParam(name = PtConstants.PAGE_ROWS, value = "每页条数", required = true, paramType = "query", dataType = "int"),
            @ApiImplicitParam(name = PtConstants.PAGE_SIDX, value = "排序字段", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = PtConstants.PAGE_SORD, value = "排序方式", paramType = "query", dataType = "String"),
    })
    @PostMapping("/page")
	public PtResult<IPage<LmsEquipCheckDetail>> getPage(LmsEquipCheckDetail lmsEquipCheckDetail) {
        return success(lmsEquipCheckDetailService.page(getPage(), lmsEquipCheckDetail));
	}

    @ApiOperation(value = "查询列表", notes = "查询数据列表")
    @PostMapping("/list")
    public PtResult<List<LmsEquipCheckDetail>> getList(LmsEquipCheckDetail lmsEquipCheckDetail) {
        return success(lmsEquipCheckDetailService.list(lmsEquipCheckDetail));
    }

    @Override
    @ApiOperation(value = "查询 id 信息", notes = "查询指定 id 的数据内容")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "主键", required = true, paramType = "path", dataType = "long"),
    })
    @GetMapping("/{id}")
    public PtResult<LmsEquipCheckDetail> get(@PathVariable("id") Long id) {
        return success(lmsEquipCheckDetailService.getById(id));
    }

    @ApiOperation(value = "仪器核查计划的修改 id 修改信息", notes = "根据指定 id 修改该条数据内容")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "主键", required = true, paramType = "path", dataType = "long"),
    })
    @PutMapping("/{id}")
	public PtResult<Boolean> edit(@PathVariable("id") Long id, LmsEquipCheckDetail lmsEquipCheckDetail) {
        lmsEquipCheckDetail.setId(id);
        return success(lmsEquipCheckDetailService.updateByIds(lmsEquipCheckDetail));
	}

    @Override
    @ApiOperation(value = "添加", notes = "添加一条数据")
    @PostMapping("/")
	public PtResult<Boolean> add(LmsEquipCheckDetail lmsEquipCheckDetail) {
        return success(lmsEquipCheckDetailService.save(lmsEquipCheckDetail));
	}

    @ApiOperation(value = "根据 ids 删除", notes = "根据 id 数组删除指定的数据内容")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "主键数组", required = true, paramType = "query", dataType = "array"),
    })
	@DeleteMapping("/")
	public PtResult<Boolean> delete(@RequestParam("ids") Long[] ids) {
        return success(lmsEquipCheckDetailService.removes(ids));
	}


    @ApiOperation(value = "上传附件", notes = "上传附件")
    @PostMapping("/uploadFile")
    public PtResult<LmsEquipFile> uploadFile(@RequestParam("file") MultipartFile file, LmsEquipCheckDetail lmsEquipCheckDetail, int type){
        return success(lmsEquipCheckDetailService.uploadFile(file, lmsEquipCheckDetail,type));
    }


    @ApiOperation(value = "下载附件", notes = "下载指定产品的产品附件")
    @GetMapping("/downLoadFile")
    public void downloadFile(Long id) {
        lmsEquipCheckDetailService.downloadFile(id, response);
    }
}
