package com.patzn.lims.file.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.patzn.lims.core.PtConstants;
import com.patzn.lims.core.api.PtResult;
import com.patzn.lims.core.web.ServiceController;
import com.patzn.lims.file.entity.LmsFileFolder;
import com.patzn.lims.file.service.ILmsFileFolderService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import java.util.List;

/**
 * <p>
 * 高校文件夹 前端控制器
 * </p>
 *
 * @author wangweidong
 * @since 2019-12-13
 */
@Api(description = "高校文件夹")
@RestController
@RequestMapping("/v1/file_folder")
public class LmsFileFolderController extends ServiceController<ILmsFileFolderService, LmsFileFolder> {


    @ApiOperation(value = "分页列表", notes = "分页查询数据列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = PtConstants.PAGE_PAGE, value = "请求数据的页码", required = true, paramType = "query", dataType = "int"),
            @ApiImplicitParam(name = PtConstants.PAGE_ROWS, value = "每页条数", required = true, paramType = "query", dataType = "int"),
            @ApiImplicitParam(name = PtConstants.PAGE_SIDX, value = "排序字段", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = PtConstants.PAGE_SORD, value = "排序方式", paramType = "query", dataType = "String"),
    })
    @PostMapping("/page")
	public PtResult<IPage<LmsFileFolder>> getPage(LmsFileFolder schoolFileFolder) {
        return success(baseService.page(getPage(), schoolFileFolder));
	}


    @ApiOperation(value = "查询列表", notes = "查询数据列表")
    @PostMapping("/list")
    public PtResult<List<LmsFileFolder>> getList(LmsFileFolder schoolFileFolder) {
        return success(baseService.list(schoolFileFolder));
    }


    @ApiOperation(value = "查询 id 信息", notes = "查询指定 id 的数据内容")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "主键", required = true, paramType = "path", dataType = "long"),
    })
    @GetMapping("/{id}")
    public PtResult<LmsFileFolder> get(@PathVariable("id") Long id) {
        return success(baseService.getById(id));
    }


    @ApiOperation(value = "根据 id 修改信息", notes = "根据指定 id 修改该条数据内容")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "主键", required = true, paramType = "path", dataType = "long"),
    })
    @PutMapping("/{id}")
	public PtResult<Boolean> edit(@PathVariable("id") Long id, LmsFileFolder schoolFileFolder) {
        schoolFileFolder.setId(id);
        return success(baseService.updateById(schoolFileFolder));
	}





    @ApiOperation(value = "添加", notes = "添加一条数据")
    @PostMapping("/")
	public PtResult<Boolean> add(LmsFileFolder schoolFileFolder) {
        return success(baseService.save(schoolFileFolder));
	}


    @ApiOperation(value = "根据 ids 删除", notes = "根据 id 数组删除指定的数据内容")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "主键数组", required = true, paramType = "query", dataType = "array"),
    })
	@DeleteMapping("/")
	public PtResult<Boolean> delete(@RequestParam("ids") Long[] ids) {
        return success(baseService.removeFolder(ids,getAccount()));
	}



    @PostMapping("/add_folder")
    public PtResult<Boolean> addFolder(LmsFileFolder fileFolder){
        return  success(baseService.addFolder(fileFolder,getAccount()));
    }

}
