package com.patzn.lims.res.service.impl;


import com.patzn.cloud.commons.api.RestAssert;
import com.patzn.lims.res.service.IExportService;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.xssf.streaming.SXSSFWorkbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.apache.poi.xwpf.usermodel.XWPFDocument;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URL;
import java.util.HashMap;
import java.util.Map;

/**
 * <p>
 * 导出相关
 * </p>
 *
 * @author wwd
 * @since 2018-06-13
 */
@Service
public class ExportServiceImpl implements IExportService {

    private final Logger logger = LoggerFactory.getLogger(ExportServiceImpl.class);


    @Override
    public String storageWord(XWPFDocument doc, String filePath) {
        try (
                ByteArrayOutputStream bos = new ByteArrayOutputStream();
                FileOutputStream out = new FileOutputStream(filePath, true)
        ) {

            doc.write(bos);
            out.write(bos.toByteArray());
        } catch (Exception e) {
            logger.error("保存 word 异常", e);
            return null;
        }
        return filePath;
    }


    @Override
    public Map<String, Object> wordToMap(String docFilePath) {

        String filePath = String.format("templates/word/model/%s", docFilePath);
        // 兼容打包 JAR 压缩情况处理
        logger.debug("doc 文件路径 {}", docFilePath);
        XWPFDocument doc = null;
        try (InputStream in = this.getClass().getClassLoader().getResourceAsStream(filePath)) {
            doc = new XWPFDocument(in);
        } catch (Exception e) {
            logger.error("word 转 map 异常", e);
        }

        Map<String, Object> map = new HashMap<>(2);
        if (null!=doc&&null != doc.getTables()) {
            map.put("table", doc.getTables().iterator());
        }
        map.put("doc", doc);
        return map;
    }


    @Override
    public String filePath(String fileName) {
        URL url = this.getClass().getClassLoader().getResource("templates/word/model/" + fileName);
        RestAssert.fail(null == url, fileName + "不存在");
        return url.getPath();
    }

    @Override
    public String fileFolderPath(String folder,String fileName) {
        URL url = this.getClass().getClassLoader().getResource("templates/word/"+folder+"/" + fileName);
        RestAssert.fail(null == url, fileName + "不存在");
        return url.getPath();
    }


    public String fileFolderPath(String fileName) {
        URL url = this.getClass().getClassLoader().getResource("templates/excel/"+fileName);
        RestAssert.fail(null == url, fileName + "不存在");
        return url.getPath();
    }

    public InputStream fileInputStreamPath(String fileName) {
        InputStream url = this.getClass().getClassLoader().getResourceAsStream("templates/excel/"+fileName);
        return url;
    }

    @Override
    public void downloadDoc(HttpServletResponse response, String fileName, XWPFDocument doc) {
        response.reset();
        response.setCharacterEncoding("UTF-8");
        response.setContentType("application/vnd.openxmlformats-officedocument.wordprocessingml.document");
        response.addHeader("Content-Disposition", String.format("attachment; filename=%s;charset=UTF-8", fileName));
        try (
                ByteArrayOutputStream bos = new ByteArrayOutputStream();
                OutputStream os = response.getOutputStream()
        ) {
            doc.write(bos);
            os.write(bos.toByteArray());
            os.flush();
        } catch (Exception e) {
            logger.error("word 转 map 异常", e);
        }
    }

    @Override
    public void downloadXls(HttpServletResponse response, String fileName, HSSFWorkbook workbook) {
        response.reset();
        response.setCharacterEncoding("UTF-8");
        response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
        response.addHeader("Content-Disposition", String.format("attachment; filename=%s;charset=UTF-8", fileName));
        try (
                ByteArrayOutputStream bos = new ByteArrayOutputStream();
                OutputStream os = response.getOutputStream()
        ) {
            workbook.write(bos);
            os.write(bos.toByteArray());
            os.flush();
        } catch (Exception e) {
            logger.error("excel 转 map 异常", e);
        }
    }

    @Override
    public void downloadXlsx(HttpServletResponse response, String fileName, XSSFWorkbook workbook) {
        response.reset();
        response.setCharacterEncoding("UTF-8");
        response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
        response.addHeader("Content-Disposition", String.format("attachment; filename=%s;charset=UTF-8", fileName));
        try (
                ByteArrayOutputStream bos = new ByteArrayOutputStream();
                OutputStream os = response.getOutputStream()
        ) {
            workbook.write(bos);
            os.write(bos.toByteArray());
            os.flush();
        } catch (Exception e) {
            logger.error("excel 转 map 异常", e);
        }
    }

    @Override
    public void downloadXlsxs(HttpServletResponse response, String fileName, SXSSFWorkbook workbook) {
        response.reset();
        response.setCharacterEncoding("UTF-8");
        response.setContentType("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet");
        response.addHeader("Content-Disposition", String.format("attachment; filename=%s;charset=UTF-8", fileName));
        try (
                ByteArrayOutputStream bos = new ByteArrayOutputStream();
                OutputStream os = response.getOutputStream()
        ) {
            workbook.write(bos);
            os.write(bos.toByteArray());
            os.flush();
        } catch (Exception e) {
            logger.error("excel 转 map 异常", e);
        }
    }

    @Override
    public XWPFDocument getXWPFDoc(String folder, String modelName) {
        FileInputStream in;
        XWPFDocument doc  = null;
        try {
            in = new FileInputStream(new File(fileFolderPath(folder,modelName)));
            doc = new XWPFDocument(in);
        } catch (Exception e1) {
            e1.printStackTrace();
        }
        return  doc;
    }

    @Override
    public XWPFDocument getXWPFDoc(InputStream io) {
        XWPFDocument doc  = null;
        try {
            doc = new XWPFDocument(io);
        } catch (Exception e1) {
            e1.printStackTrace();
        }
        return  doc;
    }

    @Override
    public XSSFWorkbook getXSSFWorkbook(String modelName){
        InputStream is;
        XSSFWorkbook xssfWorkbook=null;
        try {
            is = fileInputStreamPath(modelName);
            xssfWorkbook = new XSSFWorkbook(is);
        } catch (Exception e1) {
            e1.printStackTrace();
        }
        return xssfWorkbook;
    }

    @Override
    public SXSSFWorkbook getSXSSFWorkbook(String modelName){
        InputStream is;
        SXSSFWorkbook xssfWorkbook=null;
        try {
            is = fileInputStreamPath(modelName);
            xssfWorkbook = new SXSSFWorkbook(new XSSFWorkbook(is));
            xssfWorkbook.setCompressTempFiles(true); //压缩临时文件，很重要，否则磁盘很快就会被写满
        } catch (Exception e1) {
            e1.printStackTrace();
        }
        return xssfWorkbook;
    }

    @Override
    public XSSFWorkbook getXSSFWorkbook(InputStream io) {
        XSSFWorkbook xssfWorkbook=null;
        try {
            xssfWorkbook = new XSSFWorkbook(io);
        } catch (Exception e1) {
            e1.printStackTrace();
        }
        return xssfWorkbook;
    }


    @Override
    public HSSFWorkbook getHSSFWorkbook(String modelName){
        FileInputStream is;
        HSSFWorkbook xssfWorkbook=null;
        try {
            is = new FileInputStream(new File(fileFolderPath(modelName)));
            xssfWorkbook = new HSSFWorkbook(is);
        } catch (Exception e1) {
            e1.printStackTrace();
        }
        return xssfWorkbook;
    }
}